@@ -47,6 +47,7 @@
 struct regional;
 struct config_file;
 struct config_view;
+struct respip_set;
 
 
 /**
@@ -66,11 +67,13 @@ struct view {
 	/** rbtree node, key is name */
 	rbnode_type node;
 	/** view name.
-	 * Has to be right after rbnode_t due to pointer arithmatic in
+	 * Has to be right after rbnode_t due to pointer arithmetic in
 	 * view_create's lock protect */
 	char* name;
 	/** view specific local authority zones */
 	struct local_zones* local_zones;
+	/** response-ip configuration data for this view */
+	struct respip_set* respip_set;
 	/** Fallback to global local_zones when there is no match in the view
 	 * specific tree. 1 for yes, 0 for no */	
 	int isfirst;
@@ -123,7 +126,8 @@ void view_delete(struct view* v);
  */
 void views_print(struct views* v);
 
-/* Find a view by name.
+/**
+ * Find a view by name.
  * @param vs: views
  * @param name: name of the view we are looking for
  * @param write: 1 for obtaining write lock on found view, 0 for read lock
