@@ -278,128 +278,136 @@ extern int yydebug;
     VAR_CAPS_WHITELIST = 479,      /* VAR_CAPS_WHITELIST  */
     VAR_CACHE_MAX_NEGATIVE_TTL = 480, /* VAR_CACHE_MAX_NEGATIVE_TTL  */
     VAR_PERMIT_SMALL_HOLDDOWN = 481, /* VAR_PERMIT_SMALL_HOLDDOWN  */
-    VAR_QNAME_MINIMISATION = 482,  /* VAR_QNAME_MINIMISATION  */
-    VAR_QNAME_MINIMISATION_STRICT = 483, /* VAR_QNAME_MINIMISATION_STRICT  */
-    VAR_IP_FREEBIND = 484,         /* VAR_IP_FREEBIND  */
-    VAR_DEFINE_TAG = 485,          /* VAR_DEFINE_TAG  */
-    VAR_LOCAL_ZONE_TAG = 486,      /* VAR_LOCAL_ZONE_TAG  */
-    VAR_ACCESS_CONTROL_TAG = 487,  /* VAR_ACCESS_CONTROL_TAG  */
-    VAR_LOCAL_ZONE_OVERRIDE = 488, /* VAR_LOCAL_ZONE_OVERRIDE  */
-    VAR_ACCESS_CONTROL_TAG_ACTION = 489, /* VAR_ACCESS_CONTROL_TAG_ACTION  */
-    VAR_ACCESS_CONTROL_TAG_DATA = 490, /* VAR_ACCESS_CONTROL_TAG_DATA  */
-    VAR_VIEW = 491,                /* VAR_VIEW  */
-    VAR_ACCESS_CONTROL_VIEW = 492, /* VAR_ACCESS_CONTROL_VIEW  */
-    VAR_VIEW_FIRST = 493,          /* VAR_VIEW_FIRST  */
-    VAR_SERVE_EXPIRED = 494,       /* VAR_SERVE_EXPIRED  */
-    VAR_SERVE_EXPIRED_TTL = 495,   /* VAR_SERVE_EXPIRED_TTL  */
-    VAR_SERVE_EXPIRED_TTL_RESET = 496, /* VAR_SERVE_EXPIRED_TTL_RESET  */
-    VAR_SERVE_EXPIRED_REPLY_TTL = 497, /* VAR_SERVE_EXPIRED_REPLY_TTL  */
-    VAR_SERVE_EXPIRED_CLIENT_TIMEOUT = 498, /* VAR_SERVE_EXPIRED_CLIENT_TIMEOUT  */
-    VAR_EDE_SERVE_EXPIRED = 499,   /* VAR_EDE_SERVE_EXPIRED  */
-    VAR_SERVE_ORIGINAL_TTL = 500,  /* VAR_SERVE_ORIGINAL_TTL  */
-    VAR_FAKE_DSA = 501,            /* VAR_FAKE_DSA  */
-    VAR_FAKE_SHA1 = 502,           /* VAR_FAKE_SHA1  */
-    VAR_LOG_IDENTITY = 503,        /* VAR_LOG_IDENTITY  */
-    VAR_HIDE_TRUSTANCHOR = 504,    /* VAR_HIDE_TRUSTANCHOR  */
-    VAR_HIDE_HTTP_USER_AGENT = 505, /* VAR_HIDE_HTTP_USER_AGENT  */
-    VAR_HTTP_USER_AGENT = 506,     /* VAR_HTTP_USER_AGENT  */
-    VAR_TRUST_ANCHOR_SIGNALING = 507, /* VAR_TRUST_ANCHOR_SIGNALING  */
-    VAR_AGGRESSIVE_NSEC = 508,     /* VAR_AGGRESSIVE_NSEC  */
-    VAR_USE_SYSTEMD = 509,         /* VAR_USE_SYSTEMD  */
-    VAR_SHM_ENABLE = 510,          /* VAR_SHM_ENABLE  */
-    VAR_SHM_KEY = 511,             /* VAR_SHM_KEY  */
-    VAR_ROOT_KEY_SENTINEL = 512,   /* VAR_ROOT_KEY_SENTINEL  */
-    VAR_DNSCRYPT = 513,            /* VAR_DNSCRYPT  */
-    VAR_DNSCRYPT_ENABLE = 514,     /* VAR_DNSCRYPT_ENABLE  */
-    VAR_DNSCRYPT_PORT = 515,       /* VAR_DNSCRYPT_PORT  */
-    VAR_DNSCRYPT_PROVIDER = 516,   /* VAR_DNSCRYPT_PROVIDER  */
-    VAR_DNSCRYPT_SECRET_KEY = 517, /* VAR_DNSCRYPT_SECRET_KEY  */
-    VAR_DNSCRYPT_PROVIDER_CERT = 518, /* VAR_DNSCRYPT_PROVIDER_CERT  */
-    VAR_DNSCRYPT_PROVIDER_CERT_ROTATED = 519, /* VAR_DNSCRYPT_PROVIDER_CERT_ROTATED  */
-    VAR_DNSCRYPT_SHARED_SECRET_CACHE_SIZE = 520, /* VAR_DNSCRYPT_SHARED_SECRET_CACHE_SIZE  */
-    VAR_DNSCRYPT_SHARED_SECRET_CACHE_SLABS = 521, /* VAR_DNSCRYPT_SHARED_SECRET_CACHE_SLABS  */
-    VAR_DNSCRYPT_NONCE_CACHE_SIZE = 522, /* VAR_DNSCRYPT_NONCE_CACHE_SIZE  */
-    VAR_DNSCRYPT_NONCE_CACHE_SLABS = 523, /* VAR_DNSCRYPT_NONCE_CACHE_SLABS  */
-    VAR_PAD_RESPONSES = 524,       /* VAR_PAD_RESPONSES  */
-    VAR_PAD_RESPONSES_BLOCK_SIZE = 525, /* VAR_PAD_RESPONSES_BLOCK_SIZE  */
-    VAR_PAD_QUERIES = 526,         /* VAR_PAD_QUERIES  */
-    VAR_PAD_QUERIES_BLOCK_SIZE = 527, /* VAR_PAD_QUERIES_BLOCK_SIZE  */
-    VAR_IPSECMOD_ENABLED = 528,    /* VAR_IPSECMOD_ENABLED  */
-    VAR_IPSECMOD_HOOK = 529,       /* VAR_IPSECMOD_HOOK  */
-    VAR_IPSECMOD_IGNORE_BOGUS = 530, /* VAR_IPSECMOD_IGNORE_BOGUS  */
-    VAR_IPSECMOD_MAX_TTL = 531,    /* VAR_IPSECMOD_MAX_TTL  */
-    VAR_IPSECMOD_WHITELIST = 532,  /* VAR_IPSECMOD_WHITELIST  */
-    VAR_IPSECMOD_STRICT = 533,     /* VAR_IPSECMOD_STRICT  */
-    VAR_CACHEDB = 534,             /* VAR_CACHEDB  */
-    VAR_CACHEDB_BACKEND = 535,     /* VAR_CACHEDB_BACKEND  */
-    VAR_CACHEDB_SECRETSEED = 536,  /* VAR_CACHEDB_SECRETSEED  */
-    VAR_CACHEDB_REDISHOST = 537,   /* VAR_CACHEDB_REDISHOST  */
-    VAR_CACHEDB_REDISPORT = 538,   /* VAR_CACHEDB_REDISPORT  */
-    VAR_CACHEDB_REDISTIMEOUT = 539, /* VAR_CACHEDB_REDISTIMEOUT  */
-    VAR_CACHEDB_REDISEXPIRERECORDS = 540, /* VAR_CACHEDB_REDISEXPIRERECORDS  */
-    VAR_CACHEDB_REDISPATH = 541,   /* VAR_CACHEDB_REDISPATH  */
-    VAR_CACHEDB_REDISPASSWORD = 542, /* VAR_CACHEDB_REDISPASSWORD  */
-    VAR_CACHEDB_REDISLOGICALDB = 543, /* VAR_CACHEDB_REDISLOGICALDB  */
-    VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM = 544, /* VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM  */
-    VAR_FOR_UPSTREAM = 545,        /* VAR_FOR_UPSTREAM  */
-    VAR_AUTH_ZONE = 546,           /* VAR_AUTH_ZONE  */
-    VAR_ZONEFILE = 547,            /* VAR_ZONEFILE  */
-    VAR_MASTER = 548,              /* VAR_MASTER  */
-    VAR_URL = 549,                 /* VAR_URL  */
-    VAR_FOR_DOWNSTREAM = 550,      /* VAR_FOR_DOWNSTREAM  */
-    VAR_FALLBACK_ENABLED = 551,    /* VAR_FALLBACK_ENABLED  */
-    VAR_TLS_ADDITIONAL_PORT = 552, /* VAR_TLS_ADDITIONAL_PORT  */
-    VAR_LOW_RTT = 553,             /* VAR_LOW_RTT  */
-    VAR_LOW_RTT_PERMIL = 554,      /* VAR_LOW_RTT_PERMIL  */
-    VAR_FAST_SERVER_PERMIL = 555,  /* VAR_FAST_SERVER_PERMIL  */
-    VAR_FAST_SERVER_NUM = 556,     /* VAR_FAST_SERVER_NUM  */
-    VAR_ALLOW_NOTIFY = 557,        /* VAR_ALLOW_NOTIFY  */
-    VAR_TLS_WIN_CERT = 558,        /* VAR_TLS_WIN_CERT  */
-    VAR_TCP_CONNECTION_LIMIT = 559, /* VAR_TCP_CONNECTION_LIMIT  */
-    VAR_ANSWER_COOKIE = 560,       /* VAR_ANSWER_COOKIE  */
-    VAR_COOKIE_SECRET = 561,       /* VAR_COOKIE_SECRET  */
-    VAR_IP_RATELIMIT_COOKIE = 562, /* VAR_IP_RATELIMIT_COOKIE  */
-    VAR_FORWARD_NO_CACHE = 563,    /* VAR_FORWARD_NO_CACHE  */
-    VAR_STUB_NO_CACHE = 564,       /* VAR_STUB_NO_CACHE  */
-    VAR_LOG_SERVFAIL = 565,        /* VAR_LOG_SERVFAIL  */
-    VAR_DENY_ANY = 566,            /* VAR_DENY_ANY  */
-    VAR_UNKNOWN_SERVER_TIME_LIMIT = 567, /* VAR_UNKNOWN_SERVER_TIME_LIMIT  */
-    VAR_LOG_TAG_QUERYREPLY = 568,  /* VAR_LOG_TAG_QUERYREPLY  */
-    VAR_STREAM_WAIT_SIZE = 569,    /* VAR_STREAM_WAIT_SIZE  */
-    VAR_TLS_CIPHERS = 570,         /* VAR_TLS_CIPHERS  */
-    VAR_TLS_CIPHERSUITES = 571,    /* VAR_TLS_CIPHERSUITES  */
-    VAR_TLS_USE_SNI = 572,         /* VAR_TLS_USE_SNI  */
-    VAR_IPSET = 573,               /* VAR_IPSET  */
-    VAR_IPSET_NAME_V4 = 574,       /* VAR_IPSET_NAME_V4  */
-    VAR_IPSET_NAME_V6 = 575,       /* VAR_IPSET_NAME_V6  */
-    VAR_TLS_SESSION_TICKET_KEYS = 576, /* VAR_TLS_SESSION_TICKET_KEYS  */
-    VAR_RPZ = 577,                 /* VAR_RPZ  */
-    VAR_TAGS = 578,                /* VAR_TAGS  */
-    VAR_RPZ_ACTION_OVERRIDE = 579, /* VAR_RPZ_ACTION_OVERRIDE  */
-    VAR_RPZ_CNAME_OVERRIDE = 580,  /* VAR_RPZ_CNAME_OVERRIDE  */
-    VAR_RPZ_LOG = 581,             /* VAR_RPZ_LOG  */
-    VAR_RPZ_LOG_NAME = 582,        /* VAR_RPZ_LOG_NAME  */
-    VAR_DYNLIB = 583,              /* VAR_DYNLIB  */
-    VAR_DYNLIB_FILE = 584,         /* VAR_DYNLIB_FILE  */
-    VAR_EDNS_CLIENT_STRING = 585,  /* VAR_EDNS_CLIENT_STRING  */
-    VAR_EDNS_CLIENT_STRING_OPCODE = 586, /* VAR_EDNS_CLIENT_STRING_OPCODE  */
-    VAR_NSID = 587,                /* VAR_NSID  */
-    VAR_ZONEMD_PERMISSIVE_MODE = 588, /* VAR_ZONEMD_PERMISSIVE_MODE  */
-    VAR_ZONEMD_CHECK = 589,        /* VAR_ZONEMD_CHECK  */
-    VAR_ZONEMD_REJECT_ABSENCE = 590, /* VAR_ZONEMD_REJECT_ABSENCE  */
-    VAR_RPZ_SIGNAL_NXDOMAIN_RA = 591, /* VAR_RPZ_SIGNAL_NXDOMAIN_RA  */
-    VAR_INTERFACE_AUTOMATIC_PORTS = 592, /* VAR_INTERFACE_AUTOMATIC_PORTS  */
-    VAR_EDE = 593,                 /* VAR_EDE  */
-    VAR_INTERFACE_ACTION = 594,    /* VAR_INTERFACE_ACTION  */
-    VAR_INTERFACE_VIEW = 595,      /* VAR_INTERFACE_VIEW  */
-    VAR_INTERFACE_TAG = 596,       /* VAR_INTERFACE_TAG  */
-    VAR_INTERFACE_TAG_ACTION = 597, /* VAR_INTERFACE_TAG_ACTION  */
-    VAR_INTERFACE_TAG_DATA = 598,  /* VAR_INTERFACE_TAG_DATA  */
-    VAR_PROXY_PROTOCOL_PORT = 599, /* VAR_PROXY_PROTOCOL_PORT  */
-    VAR_STATISTICS_INHIBIT_ZERO = 600, /* VAR_STATISTICS_INHIBIT_ZERO  */
-    VAR_HARDEN_UNKNOWN_ADDITIONAL = 601, /* VAR_HARDEN_UNKNOWN_ADDITIONAL  */
-    VAR_DISABLE_EDNS_DO = 602,     /* VAR_DISABLE_EDNS_DO  */
-    VAR_CACHEDB_NO_STORE = 603     /* VAR_CACHEDB_NO_STORE  */
+    VAR_CACHE_MIN_NEGATIVE_TTL = 482, /* VAR_CACHE_MIN_NEGATIVE_TTL  */
+    VAR_QNAME_MINIMISATION = 483,  /* VAR_QNAME_MINIMISATION  */
+    VAR_QNAME_MINIMISATION_STRICT = 484, /* VAR_QNAME_MINIMISATION_STRICT  */
+    VAR_IP_FREEBIND = 485,         /* VAR_IP_FREEBIND  */
+    VAR_DEFINE_TAG = 486,          /* VAR_DEFINE_TAG  */
+    VAR_LOCAL_ZONE_TAG = 487,      /* VAR_LOCAL_ZONE_TAG  */
+    VAR_ACCESS_CONTROL_TAG = 488,  /* VAR_ACCESS_CONTROL_TAG  */
+    VAR_LOCAL_ZONE_OVERRIDE = 489, /* VAR_LOCAL_ZONE_OVERRIDE  */
+    VAR_ACCESS_CONTROL_TAG_ACTION = 490, /* VAR_ACCESS_CONTROL_TAG_ACTION  */
+    VAR_ACCESS_CONTROL_TAG_DATA = 491, /* VAR_ACCESS_CONTROL_TAG_DATA  */
+    VAR_VIEW = 492,                /* VAR_VIEW  */
+    VAR_ACCESS_CONTROL_VIEW = 493, /* VAR_ACCESS_CONTROL_VIEW  */
+    VAR_VIEW_FIRST = 494,          /* VAR_VIEW_FIRST  */
+    VAR_SERVE_EXPIRED = 495,       /* VAR_SERVE_EXPIRED  */
+    VAR_SERVE_EXPIRED_TTL = 496,   /* VAR_SERVE_EXPIRED_TTL  */
+    VAR_SERVE_EXPIRED_TTL_RESET = 497, /* VAR_SERVE_EXPIRED_TTL_RESET  */
+    VAR_SERVE_EXPIRED_REPLY_TTL = 498, /* VAR_SERVE_EXPIRED_REPLY_TTL  */
+    VAR_SERVE_EXPIRED_CLIENT_TIMEOUT = 499, /* VAR_SERVE_EXPIRED_CLIENT_TIMEOUT  */
+    VAR_EDE_SERVE_EXPIRED = 500,   /* VAR_EDE_SERVE_EXPIRED  */
+    VAR_SERVE_ORIGINAL_TTL = 501,  /* VAR_SERVE_ORIGINAL_TTL  */
+    VAR_FAKE_DSA = 502,            /* VAR_FAKE_DSA  */
+    VAR_FAKE_SHA1 = 503,           /* VAR_FAKE_SHA1  */
+    VAR_LOG_IDENTITY = 504,        /* VAR_LOG_IDENTITY  */
+    VAR_HIDE_TRUSTANCHOR = 505,    /* VAR_HIDE_TRUSTANCHOR  */
+    VAR_HIDE_HTTP_USER_AGENT = 506, /* VAR_HIDE_HTTP_USER_AGENT  */
+    VAR_HTTP_USER_AGENT = 507,     /* VAR_HTTP_USER_AGENT  */
+    VAR_TRUST_ANCHOR_SIGNALING = 508, /* VAR_TRUST_ANCHOR_SIGNALING  */
+    VAR_AGGRESSIVE_NSEC = 509,     /* VAR_AGGRESSIVE_NSEC  */
+    VAR_USE_SYSTEMD = 510,         /* VAR_USE_SYSTEMD  */
+    VAR_SHM_ENABLE = 511,          /* VAR_SHM_ENABLE  */
+    VAR_SHM_KEY = 512,             /* VAR_SHM_KEY  */
+    VAR_ROOT_KEY_SENTINEL = 513,   /* VAR_ROOT_KEY_SENTINEL  */
+    VAR_DNSCRYPT = 514,            /* VAR_DNSCRYPT  */
+    VAR_DNSCRYPT_ENABLE = 515,     /* VAR_DNSCRYPT_ENABLE  */
+    VAR_DNSCRYPT_PORT = 516,       /* VAR_DNSCRYPT_PORT  */
+    VAR_DNSCRYPT_PROVIDER = 517,   /* VAR_DNSCRYPT_PROVIDER  */
+    VAR_DNSCRYPT_SECRET_KEY = 518, /* VAR_DNSCRYPT_SECRET_KEY  */
+    VAR_DNSCRYPT_PROVIDER_CERT = 519, /* VAR_DNSCRYPT_PROVIDER_CERT  */
+    VAR_DNSCRYPT_PROVIDER_CERT_ROTATED = 520, /* VAR_DNSCRYPT_PROVIDER_CERT_ROTATED  */
+    VAR_DNSCRYPT_SHARED_SECRET_CACHE_SIZE = 521, /* VAR_DNSCRYPT_SHARED_SECRET_CACHE_SIZE  */
+    VAR_DNSCRYPT_SHARED_SECRET_CACHE_SLABS = 522, /* VAR_DNSCRYPT_SHARED_SECRET_CACHE_SLABS  */
+    VAR_DNSCRYPT_NONCE_CACHE_SIZE = 523, /* VAR_DNSCRYPT_NONCE_CACHE_SIZE  */
+    VAR_DNSCRYPT_NONCE_CACHE_SLABS = 524, /* VAR_DNSCRYPT_NONCE_CACHE_SLABS  */
+    VAR_PAD_RESPONSES = 525,       /* VAR_PAD_RESPONSES  */
+    VAR_PAD_RESPONSES_BLOCK_SIZE = 526, /* VAR_PAD_RESPONSES_BLOCK_SIZE  */
+    VAR_PAD_QUERIES = 527,         /* VAR_PAD_QUERIES  */
+    VAR_PAD_QUERIES_BLOCK_SIZE = 528, /* VAR_PAD_QUERIES_BLOCK_SIZE  */
+    VAR_IPSECMOD_ENABLED = 529,    /* VAR_IPSECMOD_ENABLED  */
+    VAR_IPSECMOD_HOOK = 530,       /* VAR_IPSECMOD_HOOK  */
+    VAR_IPSECMOD_IGNORE_BOGUS = 531, /* VAR_IPSECMOD_IGNORE_BOGUS  */
+    VAR_IPSECMOD_MAX_TTL = 532,    /* VAR_IPSECMOD_MAX_TTL  */
+    VAR_IPSECMOD_WHITELIST = 533,  /* VAR_IPSECMOD_WHITELIST  */
+    VAR_IPSECMOD_STRICT = 534,     /* VAR_IPSECMOD_STRICT  */
+    VAR_CACHEDB = 535,             /* VAR_CACHEDB  */
+    VAR_CACHEDB_BACKEND = 536,     /* VAR_CACHEDB_BACKEND  */
+    VAR_CACHEDB_SECRETSEED = 537,  /* VAR_CACHEDB_SECRETSEED  */
+    VAR_CACHEDB_REDISHOST = 538,   /* VAR_CACHEDB_REDISHOST  */
+    VAR_CACHEDB_REDISPORT = 539,   /* VAR_CACHEDB_REDISPORT  */
+    VAR_CACHEDB_REDISTIMEOUT = 540, /* VAR_CACHEDB_REDISTIMEOUT  */
+    VAR_CACHEDB_REDISEXPIRERECORDS = 541, /* VAR_CACHEDB_REDISEXPIRERECORDS  */
+    VAR_CACHEDB_REDISPATH = 542,   /* VAR_CACHEDB_REDISPATH  */
+    VAR_CACHEDB_REDISPASSWORD = 543, /* VAR_CACHEDB_REDISPASSWORD  */
+    VAR_CACHEDB_REDISLOGICALDB = 544, /* VAR_CACHEDB_REDISLOGICALDB  */
+    VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM = 545, /* VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM  */
+    VAR_FOR_UPSTREAM = 546,        /* VAR_FOR_UPSTREAM  */
+    VAR_AUTH_ZONE = 547,           /* VAR_AUTH_ZONE  */
+    VAR_ZONEFILE = 548,            /* VAR_ZONEFILE  */
+    VAR_MASTER = 549,              /* VAR_MASTER  */
+    VAR_URL = 550,                 /* VAR_URL  */
+    VAR_FOR_DOWNSTREAM = 551,      /* VAR_FOR_DOWNSTREAM  */
+    VAR_FALLBACK_ENABLED = 552,    /* VAR_FALLBACK_ENABLED  */
+    VAR_TLS_ADDITIONAL_PORT = 553, /* VAR_TLS_ADDITIONAL_PORT  */
+    VAR_LOW_RTT = 554,             /* VAR_LOW_RTT  */
+    VAR_LOW_RTT_PERMIL = 555,      /* VAR_LOW_RTT_PERMIL  */
+    VAR_FAST_SERVER_PERMIL = 556,  /* VAR_FAST_SERVER_PERMIL  */
+    VAR_FAST_SERVER_NUM = 557,     /* VAR_FAST_SERVER_NUM  */
+    VAR_ALLOW_NOTIFY = 558,        /* VAR_ALLOW_NOTIFY  */
+    VAR_TLS_WIN_CERT = 559,        /* VAR_TLS_WIN_CERT  */
+    VAR_TCP_CONNECTION_LIMIT = 560, /* VAR_TCP_CONNECTION_LIMIT  */
+    VAR_ANSWER_COOKIE = 561,       /* VAR_ANSWER_COOKIE  */
+    VAR_COOKIE_SECRET = 562,       /* VAR_COOKIE_SECRET  */
+    VAR_IP_RATELIMIT_COOKIE = 563, /* VAR_IP_RATELIMIT_COOKIE  */
+    VAR_FORWARD_NO_CACHE = 564,    /* VAR_FORWARD_NO_CACHE  */
+    VAR_STUB_NO_CACHE = 565,       /* VAR_STUB_NO_CACHE  */
+    VAR_LOG_SERVFAIL = 566,        /* VAR_LOG_SERVFAIL  */
+    VAR_DENY_ANY = 567,            /* VAR_DENY_ANY  */
+    VAR_UNKNOWN_SERVER_TIME_LIMIT = 568, /* VAR_UNKNOWN_SERVER_TIME_LIMIT  */
+    VAR_LOG_TAG_QUERYREPLY = 569,  /* VAR_LOG_TAG_QUERYREPLY  */
+    VAR_DISCARD_TIMEOUT = 570,     /* VAR_DISCARD_TIMEOUT  */
+    VAR_WAIT_LIMIT = 571,          /* VAR_WAIT_LIMIT  */
+    VAR_WAIT_LIMIT_COOKIE = 572,   /* VAR_WAIT_LIMIT_COOKIE  */
+    VAR_WAIT_LIMIT_NETBLOCK = 573, /* VAR_WAIT_LIMIT_NETBLOCK  */
+    VAR_WAIT_LIMIT_COOKIE_NETBLOCK = 574, /* VAR_WAIT_LIMIT_COOKIE_NETBLOCK  */
+    VAR_STREAM_WAIT_SIZE = 575,    /* VAR_STREAM_WAIT_SIZE  */
+    VAR_TLS_CIPHERS = 576,         /* VAR_TLS_CIPHERS  */
+    VAR_TLS_CIPHERSUITES = 577,    /* VAR_TLS_CIPHERSUITES  */
+    VAR_TLS_USE_SNI = 578,         /* VAR_TLS_USE_SNI  */
+    VAR_IPSET = 579,               /* VAR_IPSET  */
+    VAR_IPSET_NAME_V4 = 580,       /* VAR_IPSET_NAME_V4  */
+    VAR_IPSET_NAME_V6 = 581,       /* VAR_IPSET_NAME_V6  */
+    VAR_TLS_SESSION_TICKET_KEYS = 582, /* VAR_TLS_SESSION_TICKET_KEYS  */
+    VAR_RPZ = 583,                 /* VAR_RPZ  */
+    VAR_TAGS = 584,                /* VAR_TAGS  */
+    VAR_RPZ_ACTION_OVERRIDE = 585, /* VAR_RPZ_ACTION_OVERRIDE  */
+    VAR_RPZ_CNAME_OVERRIDE = 586,  /* VAR_RPZ_CNAME_OVERRIDE  */
+    VAR_RPZ_LOG = 587,             /* VAR_RPZ_LOG  */
+    VAR_RPZ_LOG_NAME = 588,        /* VAR_RPZ_LOG_NAME  */
+    VAR_DYNLIB = 589,              /* VAR_DYNLIB  */
+    VAR_DYNLIB_FILE = 590,         /* VAR_DYNLIB_FILE  */
+    VAR_EDNS_CLIENT_STRING = 591,  /* VAR_EDNS_CLIENT_STRING  */
+    VAR_EDNS_CLIENT_STRING_OPCODE = 592, /* VAR_EDNS_CLIENT_STRING_OPCODE  */
+    VAR_NSID = 593,                /* VAR_NSID  */
+    VAR_ZONEMD_PERMISSIVE_MODE = 594, /* VAR_ZONEMD_PERMISSIVE_MODE  */
+    VAR_ZONEMD_CHECK = 595,        /* VAR_ZONEMD_CHECK  */
+    VAR_ZONEMD_REJECT_ABSENCE = 596, /* VAR_ZONEMD_REJECT_ABSENCE  */
+    VAR_RPZ_SIGNAL_NXDOMAIN_RA = 597, /* VAR_RPZ_SIGNAL_NXDOMAIN_RA  */
+    VAR_INTERFACE_AUTOMATIC_PORTS = 598, /* VAR_INTERFACE_AUTOMATIC_PORTS  */
+    VAR_EDE = 599,                 /* VAR_EDE  */
+    VAR_INTERFACE_ACTION = 600,    /* VAR_INTERFACE_ACTION  */
+    VAR_INTERFACE_VIEW = 601,      /* VAR_INTERFACE_VIEW  */
+    VAR_INTERFACE_TAG = 602,       /* VAR_INTERFACE_TAG  */
+    VAR_INTERFACE_TAG_ACTION = 603, /* VAR_INTERFACE_TAG_ACTION  */
+    VAR_INTERFACE_TAG_DATA = 604,  /* VAR_INTERFACE_TAG_DATA  */
+    VAR_PROXY_PROTOCOL_PORT = 605, /* VAR_PROXY_PROTOCOL_PORT  */
+    VAR_STATISTICS_INHIBIT_ZERO = 606, /* VAR_STATISTICS_INHIBIT_ZERO  */
+    VAR_HARDEN_UNKNOWN_ADDITIONAL = 607, /* VAR_HARDEN_UNKNOWN_ADDITIONAL  */
+    VAR_DISABLE_EDNS_DO = 608,     /* VAR_DISABLE_EDNS_DO  */
+    VAR_CACHEDB_NO_STORE = 609,    /* VAR_CACHEDB_NO_STORE  */
+    VAR_LOG_DESTADDR = 610,        /* VAR_LOG_DESTADDR  */
+    VAR_CACHEDB_CHECK_WHEN_SERVE_EXPIRED = 611 /* VAR_CACHEDB_CHECK_WHEN_SERVE_EXPIRED  */
   };
   typedef enum yytokentype yytoken_kind_t;
 #endif
@@ -632,128 +640,136 @@ extern int yydebug;
 #define VAR_CAPS_WHITELIST 479
 #define VAR_CACHE_MAX_NEGATIVE_TTL 480
 #define VAR_PERMIT_SMALL_HOLDDOWN 481
-#define VAR_QNAME_MINIMISATION 482
-#define VAR_QNAME_MINIMISATION_STRICT 483
-#define VAR_IP_FREEBIND 484
-#define VAR_DEFINE_TAG 485
-#define VAR_LOCAL_ZONE_TAG 486
-#define VAR_ACCESS_CONTROL_TAG 487
-#define VAR_LOCAL_ZONE_OVERRIDE 488
-#define VAR_ACCESS_CONTROL_TAG_ACTION 489
-#define VAR_ACCESS_CONTROL_TAG_DATA 490
-#define VAR_VIEW 491
-#define VAR_ACCESS_CONTROL_VIEW 492
-#define VAR_VIEW_FIRST 493
-#define VAR_SERVE_EXPIRED 494
-#define VAR_SERVE_EXPIRED_TTL 495
-#define VAR_SERVE_EXPIRED_TTL_RESET 496
-#define VAR_SERVE_EXPIRED_REPLY_TTL 497
-#define VAR_SERVE_EXPIRED_CLIENT_TIMEOUT 498
-#define VAR_EDE_SERVE_EXPIRED 499
-#define VAR_SERVE_ORIGINAL_TTL 500
-#define VAR_FAKE_DSA 501
-#define VAR_FAKE_SHA1 502
-#define VAR_LOG_IDENTITY 503
-#define VAR_HIDE_TRUSTANCHOR 504
-#define VAR_HIDE_HTTP_USER_AGENT 505
-#define VAR_HTTP_USER_AGENT 506
-#define VAR_TRUST_ANCHOR_SIGNALING 507
-#define VAR_AGGRESSIVE_NSEC 508
-#define VAR_USE_SYSTEMD 509
-#define VAR_SHM_ENABLE 510
-#define VAR_SHM_KEY 511
-#define VAR_ROOT_KEY_SENTINEL 512
-#define VAR_DNSCRYPT 513
-#define VAR_DNSCRYPT_ENABLE 514
-#define VAR_DNSCRYPT_PORT 515
-#define VAR_DNSCRYPT_PROVIDER 516
-#define VAR_DNSCRYPT_SECRET_KEY 517
-#define VAR_DNSCRYPT_PROVIDER_CERT 518
-#define VAR_DNSCRYPT_PROVIDER_CERT_ROTATED 519
-#define VAR_DNSCRYPT_SHARED_SECRET_CACHE_SIZE 520
-#define VAR_DNSCRYPT_SHARED_SECRET_CACHE_SLABS 521
-#define VAR_DNSCRYPT_NONCE_CACHE_SIZE 522
-#define VAR_DNSCRYPT_NONCE_CACHE_SLABS 523
-#define VAR_PAD_RESPONSES 524
-#define VAR_PAD_RESPONSES_BLOCK_SIZE 525
-#define VAR_PAD_QUERIES 526
-#define VAR_PAD_QUERIES_BLOCK_SIZE 527
-#define VAR_IPSECMOD_ENABLED 528
-#define VAR_IPSECMOD_HOOK 529
-#define VAR_IPSECMOD_IGNORE_BOGUS 530
-#define VAR_IPSECMOD_MAX_TTL 531
-#define VAR_IPSECMOD_WHITELIST 532
-#define VAR_IPSECMOD_STRICT 533
-#define VAR_CACHEDB 534
-#define VAR_CACHEDB_BACKEND 535
-#define VAR_CACHEDB_SECRETSEED 536
-#define VAR_CACHEDB_REDISHOST 537
-#define VAR_CACHEDB_REDISPORT 538
-#define VAR_CACHEDB_REDISTIMEOUT 539
-#define VAR_CACHEDB_REDISEXPIRERECORDS 540
-#define VAR_CACHEDB_REDISPATH 541
-#define VAR_CACHEDB_REDISPASSWORD 542
-#define VAR_CACHEDB_REDISLOGICALDB 543
-#define VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM 544
-#define VAR_FOR_UPSTREAM 545
-#define VAR_AUTH_ZONE 546
-#define VAR_ZONEFILE 547
-#define VAR_MASTER 548
-#define VAR_URL 549
-#define VAR_FOR_DOWNSTREAM 550
-#define VAR_FALLBACK_ENABLED 551
-#define VAR_TLS_ADDITIONAL_PORT 552
-#define VAR_LOW_RTT 553
-#define VAR_LOW_RTT_PERMIL 554
-#define VAR_FAST_SERVER_PERMIL 555
-#define VAR_FAST_SERVER_NUM 556
-#define VAR_ALLOW_NOTIFY 557
-#define VAR_TLS_WIN_CERT 558
-#define VAR_TCP_CONNECTION_LIMIT 559
-#define VAR_ANSWER_COOKIE 560
-#define VAR_COOKIE_SECRET 561
-#define VAR_IP_RATELIMIT_COOKIE 562
-#define VAR_FORWARD_NO_CACHE 563
-#define VAR_STUB_NO_CACHE 564
-#define VAR_LOG_SERVFAIL 565
-#define VAR_DENY_ANY 566
-#define VAR_UNKNOWN_SERVER_TIME_LIMIT 567
-#define VAR_LOG_TAG_QUERYREPLY 568
-#define VAR_STREAM_WAIT_SIZE 569
-#define VAR_TLS_CIPHERS 570
-#define VAR_TLS_CIPHERSUITES 571
-#define VAR_TLS_USE_SNI 572
-#define VAR_IPSET 573
-#define VAR_IPSET_NAME_V4 574
-#define VAR_IPSET_NAME_V6 575
-#define VAR_TLS_SESSION_TICKET_KEYS 576
-#define VAR_RPZ 577
-#define VAR_TAGS 578
-#define VAR_RPZ_ACTION_OVERRIDE 579
-#define VAR_RPZ_CNAME_OVERRIDE 580
-#define VAR_RPZ_LOG 581
-#define VAR_RPZ_LOG_NAME 582
-#define VAR_DYNLIB 583
-#define VAR_DYNLIB_FILE 584
-#define VAR_EDNS_CLIENT_STRING 585
-#define VAR_EDNS_CLIENT_STRING_OPCODE 586
-#define VAR_NSID 587
-#define VAR_ZONEMD_PERMISSIVE_MODE 588
-#define VAR_ZONEMD_CHECK 589
-#define VAR_ZONEMD_REJECT_ABSENCE 590
-#define VAR_RPZ_SIGNAL_NXDOMAIN_RA 591
-#define VAR_INTERFACE_AUTOMATIC_PORTS 592
-#define VAR_EDE 593
-#define VAR_INTERFACE_ACTION 594
-#define VAR_INTERFACE_VIEW 595
-#define VAR_INTERFACE_TAG 596
-#define VAR_INTERFACE_TAG_ACTION 597
-#define VAR_INTERFACE_TAG_DATA 598
-#define VAR_PROXY_PROTOCOL_PORT 599
-#define VAR_STATISTICS_INHIBIT_ZERO 600
-#define VAR_HARDEN_UNKNOWN_ADDITIONAL 601
-#define VAR_DISABLE_EDNS_DO 602
-#define VAR_CACHEDB_NO_STORE 603
+#define VAR_CACHE_MIN_NEGATIVE_TTL 482
+#define VAR_QNAME_MINIMISATION 483
+#define VAR_QNAME_MINIMISATION_STRICT 484
+#define VAR_IP_FREEBIND 485
+#define VAR_DEFINE_TAG 486
+#define VAR_LOCAL_ZONE_TAG 487
+#define VAR_ACCESS_CONTROL_TAG 488
+#define VAR_LOCAL_ZONE_OVERRIDE 489
+#define VAR_ACCESS_CONTROL_TAG_ACTION 490
+#define VAR_ACCESS_CONTROL_TAG_DATA 491
+#define VAR_VIEW 492
+#define VAR_ACCESS_CONTROL_VIEW 493
+#define VAR_VIEW_FIRST 494
+#define VAR_SERVE_EXPIRED 495
+#define VAR_SERVE_EXPIRED_TTL 496
+#define VAR_SERVE_EXPIRED_TTL_RESET 497
+#define VAR_SERVE_EXPIRED_REPLY_TTL 498
+#define VAR_SERVE_EXPIRED_CLIENT_TIMEOUT 499
+#define VAR_EDE_SERVE_EXPIRED 500
+#define VAR_SERVE_ORIGINAL_TTL 501
+#define VAR_FAKE_DSA 502
+#define VAR_FAKE_SHA1 503
+#define VAR_LOG_IDENTITY 504
+#define VAR_HIDE_TRUSTANCHOR 505
+#define VAR_HIDE_HTTP_USER_AGENT 506
+#define VAR_HTTP_USER_AGENT 507
+#define VAR_TRUST_ANCHOR_SIGNALING 508
+#define VAR_AGGRESSIVE_NSEC 509
+#define VAR_USE_SYSTEMD 510
+#define VAR_SHM_ENABLE 511
+#define VAR_SHM_KEY 512
+#define VAR_ROOT_KEY_SENTINEL 513
+#define VAR_DNSCRYPT 514
+#define VAR_DNSCRYPT_ENABLE 515
+#define VAR_DNSCRYPT_PORT 516
+#define VAR_DNSCRYPT_PROVIDER 517
+#define VAR_DNSCRYPT_SECRET_KEY 518
+#define VAR_DNSCRYPT_PROVIDER_CERT 519
+#define VAR_DNSCRYPT_PROVIDER_CERT_ROTATED 520
+#define VAR_DNSCRYPT_SHARED_SECRET_CACHE_SIZE 521
+#define VAR_DNSCRYPT_SHARED_SECRET_CACHE_SLABS 522
+#define VAR_DNSCRYPT_NONCE_CACHE_SIZE 523
+#define VAR_DNSCRYPT_NONCE_CACHE_SLABS 524
+#define VAR_PAD_RESPONSES 525
+#define VAR_PAD_RESPONSES_BLOCK_SIZE 526
+#define VAR_PAD_QUERIES 527
+#define VAR_PAD_QUERIES_BLOCK_SIZE 528
+#define VAR_IPSECMOD_ENABLED 529
+#define VAR_IPSECMOD_HOOK 530
+#define VAR_IPSECMOD_IGNORE_BOGUS 531
+#define VAR_IPSECMOD_MAX_TTL 532
+#define VAR_IPSECMOD_WHITELIST 533
+#define VAR_IPSECMOD_STRICT 534
+#define VAR_CACHEDB 535
+#define VAR_CACHEDB_BACKEND 536
+#define VAR_CACHEDB_SECRETSEED 537
+#define VAR_CACHEDB_REDISHOST 538
+#define VAR_CACHEDB_REDISPORT 539
+#define VAR_CACHEDB_REDISTIMEOUT 540
+#define VAR_CACHEDB_REDISEXPIRERECORDS 541
+#define VAR_CACHEDB_REDISPATH 542
+#define VAR_CACHEDB_REDISPASSWORD 543
+#define VAR_CACHEDB_REDISLOGICALDB 544
+#define VAR_UDP_UPSTREAM_WITHOUT_DOWNSTREAM 545
+#define VAR_FOR_UPSTREAM 546
+#define VAR_AUTH_ZONE 547
+#define VAR_ZONEFILE 548
+#define VAR_MASTER 549
+#define VAR_URL 550
+#define VAR_FOR_DOWNSTREAM 551
+#define VAR_FALLBACK_ENABLED 552
+#define VAR_TLS_ADDITIONAL_PORT 553
+#define VAR_LOW_RTT 554
+#define VAR_LOW_RTT_PERMIL 555
+#define VAR_FAST_SERVER_PERMIL 556
+#define VAR_FAST_SERVER_NUM 557
+#define VAR_ALLOW_NOTIFY 558
+#define VAR_TLS_WIN_CERT 559
+#define VAR_TCP_CONNECTION_LIMIT 560
+#define VAR_ANSWER_COOKIE 561
+#define VAR_COOKIE_SECRET 562
+#define VAR_IP_RATELIMIT_COOKIE 563
+#define VAR_FORWARD_NO_CACHE 564
+#define VAR_STUB_NO_CACHE 565
+#define VAR_LOG_SERVFAIL 566
+#define VAR_DENY_ANY 567
+#define VAR_UNKNOWN_SERVER_TIME_LIMIT 568
+#define VAR_LOG_TAG_QUERYREPLY 569
+#define VAR_DISCARD_TIMEOUT 570
+#define VAR_WAIT_LIMIT 571
+#define VAR_WAIT_LIMIT_COOKIE 572
+#define VAR_WAIT_LIMIT_NETBLOCK 573
+#define VAR_WAIT_LIMIT_COOKIE_NETBLOCK 574
+#define VAR_STREAM_WAIT_SIZE 575
+#define VAR_TLS_CIPHERS 576
+#define VAR_TLS_CIPHERSUITES 577
+#define VAR_TLS_USE_SNI 578
+#define VAR_IPSET 579
+#define VAR_IPSET_NAME_V4 580
+#define VAR_IPSET_NAME_V6 581
+#define VAR_TLS_SESSION_TICKET_KEYS 582
+#define VAR_RPZ 583
+#define VAR_TAGS 584
+#define VAR_RPZ_ACTION_OVERRIDE 585
+#define VAR_RPZ_CNAME_OVERRIDE 586
+#define VAR_RPZ_LOG 587
+#define VAR_RPZ_LOG_NAME 588
+#define VAR_DYNLIB 589
+#define VAR_DYNLIB_FILE 590
+#define VAR_EDNS_CLIENT_STRING 591
+#define VAR_EDNS_CLIENT_STRING_OPCODE 592
+#define VAR_NSID 593
+#define VAR_ZONEMD_PERMISSIVE_MODE 594
+#define VAR_ZONEMD_CHECK 595
+#define VAR_ZONEMD_REJECT_ABSENCE 596
+#define VAR_RPZ_SIGNAL_NXDOMAIN_RA 597
+#define VAR_INTERFACE_AUTOMATIC_PORTS 598
+#define VAR_EDE 599
+#define VAR_INTERFACE_ACTION 600
+#define VAR_INTERFACE_VIEW 601
+#define VAR_INTERFACE_TAG 602
+#define VAR_INTERFACE_TAG_ACTION 603
+#define VAR_INTERFACE_TAG_DATA 604
+#define VAR_PROXY_PROTOCOL_PORT 605
+#define VAR_STATISTICS_INHIBIT_ZERO 606
+#define VAR_HARDEN_UNKNOWN_ADDITIONAL 607
+#define VAR_DISABLE_EDNS_DO 608
+#define VAR_CACHEDB_NO_STORE 609
+#define VAR_LOG_DESTADDR 610
+#define VAR_CACHEDB_CHECK_WHEN_SERVE_EXPIRED 611
 
 /* Value type.  */
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
@@ -763,7 +779,7 @@ union YYSTYPE
 
 	char*	str;
 
-#line 767 "util/configparser.h"
+#line 783 "util/configparser.h"
 
 };
 typedef union YYSTYPE YYSTYPE;
